/***************************************************************************
 *   Copyright (C) 2004, 2006 Symbian Software Ltd.                        *
 *   All rights reserved.                                                  *
 *   Copyright (C) 2007-2011 Mike Kinghan, imk@burroingroingjoing.com      *
 *   All rights reserved.                                                  *
 *                                                                         *
 *   Contributed originally by Mike Kinghan, imk@burroingroingjoing.com    *
 *                                                                         *
 *   Redistribution and use in source and binary forms, with or without    *
 *   modification, are permitted provided that the following conditions    *
 *   are met:                                                              *
 *                                                                         *
 *   Redistributions of source code must retain the above copyright        *
 *   notice, this list of conditions and the following disclaimer.         *
 *                                                                         *
 *   Redistributions in binary form must reproduce the above copyright     *
 *   notice, this list of conditions and the following disclaimer in the   *
 *   documentation and/or other materials provided with the distribution.  *
 *                                                                         *
 *   Neither the name of Symbian Software Ltd. nor the names of its        *
 *   contributors may be used to endorse or promote products derived from  *
 *   this software without specific prior written permission.              *
 *                                                                         *
 *   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS   *
 *   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT     *
 *   LIMITED TO, THE IMPLIED WARRANTIES OF  MERCHANTABILITY AND FITNESS    *
 *   FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE        *
 *   COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,   *
 *   INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,  *
 *   BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS *
 *   OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED    *
 *   AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,*
 *   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF *
 *   THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH  *
 *   DAMAGE.                                                               *
 *                                                                         *
 **************************************************************************/
#include "line_edit.h"
#include "io.h"
#include "symbol.h"
#include <cstring>
#include <algorithm>
#include <cassert>

using namespace std;

/*!
    \file line_edit.cpp
    This file implements the \e line_edit class
*/

char*	line_edit::_keyword_ = NULL;
bool line_edit::_ops_cut_ = false;
size_t line_edit::_parens_deleted_ = 0;

void line_edit::tail_edit(char *where, const char *what)
{
	assert(io::line_start() <= where && where < io::line_end());
	io::wip_line().erase(where - io::line_start());
	io::wip_line().append(what);
}

void line_edit::keywordedit(const char *replacement)
{
	tail_edit(_keyword_,replacement);
	++line_despatch::lines_changed();
	line_despatch::print();
}

void line_edit::delete_paren(size_t lparen_off, size_t rparen_off)
{
	char * lparen = io::line_start() + lparen_off;
	char * rparen = io::line_start() + rparen_off;
	for( ; *lparen != '(' && lparen < rparen; ++lparen) {}
	for( ; *rparen != ')' && rparen > lparen; --rparen) {}
	if (*lparen == '(' && *rparen == ')' ) {
		if ((!symbol::is_valid_char(lparen[-1]) ||
		     !symbol::is_valid_char(lparen[1]))
		    &&
		    (!symbol::is_valid_char(rparen[-1]) ||
		     !symbol::is_valid_char(rparen[1]))) {
			*lparen = DELETEABLE_LPAREN;
			*rparen = DELETEABLE_RPAREN;
			++_parens_deleted_;
		}
	}
}

void line_edit::restore_paren(size_t lparen_off, size_t rparen_off)
{
	char * lparen = io::line_start() + lparen_off;
	char * rparen = io::line_start() + rparen_off;
	for( ; *lparen != DELETEABLE_LPAREN && lparen < rparen; ++lparen) {}
	for( ; *rparen != DELETEABLE_RPAREN && rparen > lparen; --rparen) {}
	if (*lparen == DELETEABLE_LPAREN && *rparen == DELETEABLE_RPAREN ) {
		*lparen = '(';
		*rparen = ')';
		--_parens_deleted_;
	}
}

void line_edit::restore_all_paren()
{
	char *buf = io::line_start();
	for (	; *buf; ++buf) {
		if (*buf == DELETEABLE_LPAREN) {
			*buf = '(';
		} else if (*buf == DELETEABLE_RPAREN) {
			*buf = ')';
		}
	};
}

void line_edit::cut_text(size_t start, size_t end)
{
	string::iterator s(io::wip_line().begin() + start);
	string::iterator e(io::wip_line().begin() + end);
	fill(s,e,DELETEABLE_CHAR);
	/* Never delete the terminal newline in directive*/
	char *last = &e[-1];
	if (!last[1]) {
		*last = '\n';
	}
	_ops_cut_ = true;

}

void line_edit::truncate_line(size_t off)
{
	assert(off < io::wip_line().length());
	io::wip_line().erase(off);

}

/* EOF*/
