/***************************************************************************
 *   Copyright (C) 2007-2011 Mike Kinghan, imk@burroingroingjoing.com      *
 *   All rights reserved.                                                  *
 *                                                                         *
 *   Redistribution and use in source and binary forms, with or without    *
 *   modification, are permitted provided that the following conditions    *
 *   are met:                                                              *
 *                                                                         *
 *   Redistributions of source code must retain the above copyright        *
 *   notice, this list of conditions and the following disclaimer.         *
 *                                                                         *
 *   Redistributions in binary form must reproduce the above copyright     *
 *   notice, this list of conditions and the following disclaimer in the   *
 *   documentation and/or other materials provided with the distribution.  *
 *                                                                         *
 *   Neither the name of Symbian Software Ltd. nor the names of its        *
 *   contributors may be used to endorse or promote products derived from  *
 *   this software without specific prior written permission.              *
 *                                                                         *
 *   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS   *
 *   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT     *
 *   LIMITED TO, THE IMPLIED WARRANTIES OF  MERCHANTABILITY AND FITNESS    *
 *   FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE        *
 *   COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,   *
 *   INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,  *
 *   BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS *
 *   OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED    *
 *   AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,*
 *   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF *
 *   THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH  *
 *   DAMAGE.                                                               *
 *                                                                         *
 **************************************************************************/

#include "line_select.h"
#include "directive.h"
#include "options.h"
#include "if_control.h"
#include "get_options.h"

/*! \file line_select.cpp
    This file implements the class \e line_select
*/

bool line_select::_eligible_;
bool line_select::_dropping_;

void line_select::set_dropping()
{
	_dropping_ = if_control::dead_line() ^ options::complement();
}

void line_select::set_eligible(directive_type directive)
{
	bool verdict;
	switch(options::get_command()) {
	case CMD_SYMBOLS:
		switch(directive) {
		case HASH_IFDEF:
		case HASH_IFNDEF:
		case HASH_IF:
			verdict = options::list_symbols_in_ifs();
			break;
		case HASH_DEFINE:
			verdict = options::list_symbols_in_defs();
			break;
		case HASH_UNDEF:
			verdict = options::list_symbols_in_undefs();
			break;
		case HASH_INCLUDE:
			verdict = options::list_symbols_in_includes();
			break;
		case HASH_UNKNOWN:
			verdict = false;
		default:
			;
			verdict = true;
		}
		break;
	case CMD_INCLUDES:
	case CMD_DEFS:
	case CMD_PRAGMAS:
	case CMD_ERRORS:
	case CMD_DIRECTIVES:
		verdict = true;
		break;
	default:
		verdict = false;
	}
	if (verdict) {
		if (options::list_only_inactive()) {
			verdict = _dropping_ && !if_control::is_unconditional_line();
		} else if (options::list_only_active()) {
			verdict = !_dropping_ && if_control::is_unconditional_line();
		}
	}
	_eligible_ = verdict;
}

bool line_select::eligible(int cmd_code)
{
	int cur_cmd = options::get_command();
	if (cur_cmd == CMD_DIRECTIVES) {
		return _eligible_ &&
		       (cmd_code == CMD_INCLUDES ||
		        cmd_code == CMD_DEFS ||
		        cmd_code == CMD_PRAGMAS ||
		        cmd_code == CMD_ERRORS);
	}
	return _eligible_ && cur_cmd == cmd_code;
}

/* EOF */
